package net.yakiboo.yakito;

import java.io.File;
import java.io.IOException;
import java.util.List;

import net.yakiboo.yakito.config.YakiToConfig;
import net.yakiboo.yakito.io.YakiToOutput;

import it.sauronsoftware.jave.Encoder;
import it.sauronsoftware.jave.EncoderException;
import it.sauronsoftware.jave.EncoderProgressListener;
import it.sauronsoftware.jave.EncodingAttributes;
import it.sauronsoftware.jave.FFMPEGExecutor;
import it.sauronsoftware.jave.InputFormatException;
import it.sauronsoftware.jave.MultimediaInfo;

/**
 * Hilo de recodificacion. Los objetos de esta clase son threads de conversion
 * independientes (uno por archivo multimedia).
 * 
 * @author Sergio Garcia Mondaray
 * 
 */
public class EncodeThread extends Thread {

	EncodingAttributes atts;
	File source, target;
	FFMPEGExecutor executor;
	Process p;
	YakiToProgress progress;
	Encoder enc;
	EncodeThread next;
	boolean concurrent;
	CustomThread parent;
	List<String> extraArgums;
	YakiToConfig config = null;
	boolean killed = false;

	/**
	 * Constructor de la clase
	 * 
	 * @param atts
	 *            Atributos de conversion
	 * @param source
	 *            Archivo multimedia original
	 * @param target
	 *            Archivo de destino (sera creado o sustituido)
	 * @param progress
	 *            Listener de progreso que debe recibir las notificaciones del
	 *            progreso de conversion
	 * @param parent
	 *            Hilo padre al que se le notificara cuando termine la
	 *            conversion
	 * @param extraArgums
	 *            Argumentos adicionales de conversion para FFMPEG
	 * @param config
	 *            Configuracion de YakiTo
	 */
	public EncodeThread(EncodingAttributes atts, File source, File target,
			YakiToProgress progress, CustomThread parent,
			List<String> extraArgums, YakiToConfig config) {
		this.atts = atts;
		this.source = source;
		this.target = target;
		this.progress = progress;
		this.parent = parent;
		this.extraArgums = extraArgums;
		this.config = config;
	}

	@Override
	public void run() {
		try {
			final YakiToProgressListener listener = new YakiToProgressListener(
					progress, source, next, parent);
			enc = new Encoder();
			executor = enc.getFFMPEGExecutor();
			for (String s : extraArgums) {
				executor.addArgument(s);
			}
			enc.encode(source, target, atts, listener);
			YakiToOutput.message("Ejecutado: " + executor.getCommand());
		} catch (IllegalArgumentException e) {
			// TODO Cambiar este error por algo mas concreto para este caso
			YakiToOutput
					.error("No se puede realizar la conversión. Es posible que algún argumento sea incorrecto.");
			YakiToOutput.errorDialog(
					config.getLanguage().getValueOf(
							"dialog_input_format_exception_title"),
					config.getLanguage().getValueOf(
							"dialog_input_format_exception_prefix")
							+ "\n"
							+ config.getLanguage().getValueOf(
									"dialog_input_format_exception_suffix"));
			progress.stopConversion();
		} catch (InputFormatException e) {
			YakiToOutput
					.error("No se puede realizar la conversión. Es posible que algún argumento sea incorrecto.");
			YakiToOutput.errorDialog(
					config.getLanguage().getValueOf(
							"dialog_input_format_exception_title"),
					config.getLanguage().getValueOf(
							"dialog_input_format_exception_prefix")
							+ "\n"
							+ config.getLanguage().getValueOf(
									"dialog_input_format_exception_suffix"));
			progress.stopConversion();
		} catch (EncoderException e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			if (!killed) {
				YakiToOutput
						.error("No se puede realizar la conversión. Uno o más archivos multimedia son incompatibles con esta versión de YakiTo. Quizá alguno de los archivos contenga más de 5 canales de audio (por ejemplo, 5.1)");
				YakiToOutput.errorDialog(
						config.getLanguage().getValueOf(
								"dialog_encoding_error_title"),
						config.getLanguage().getValueOf(
								"dialog_encoding_error_prefix")
								+ "\n"
								+ config.getLanguage().getValueOf(
										"dialog_encoding_error_suffix"));
				killed = false;
			}
			progress.stopConversion();
		}
	}

	/**
	 * Duerme el hilo de conversion
	 */
	private synchronized void toSleep() {
		try {
			this.wait();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Pausa el proceso del hilo
	 */
	public void pause() {
		System.out.println("Se ha llamado a pause()");
		// TODO Completar el metodo
	}

	/**
	 * Reanuda el proceso de conversion
	 */
	public void cont() {
		System.out.println("Se ha llamado a resume()");
		// TODO Completar el metodo
	}

	/**
	 * Mata el hilo de conversion
	 */
	public void kill() {
		killed = true;
		// System.out.println("Se ha llamado a kill()");
		if (executor != null)
			executor.destroy();
	}

}

/**
 * Clase de escucha del proceso de conversion. Recibe los avisos de progreso y
 * notifica a la clase encargada de la representacion visual.
 * 
 * @author Sergio Garcia Mondaray
 * 
 */
class YakiToProgressListener implements EncoderProgressListener {

	File f;
	YakiToProgress progress;
	EncodeThread next;
	CustomThread parent;

	/**
	 * Constructor de la clase
	 * 
	 * @param progress
	 *            Objeto encargado de la representacion visual del progreso de
	 *            conversion
	 * @param f
	 *            Archivo que se esta convirtiendo
	 * @param next
	 *            Siguiente hilo de conversion en orden secuencial
	 * @param parent
	 *            Hilo padre de la conversion
	 */
	public YakiToProgressListener(YakiToProgress progress, File f,
			EncodeThread next, CustomThread parent) {
		this.f = f;
		this.progress = progress;
		this.next = next;
		this.parent = parent;
	}

	/**
	 * Recibe un mensaje del motor de conversion
	 */
	public void message(String s) {
		// System.out.println("Encoder message >> " + s);
		if (!s.contains("[q]"))
			YakiToOutput.message("Encoder >> " + s);
	}

	/**
	 * Recibe el progreso que lleva la conversion del archivo
	 */
	public void progress(int i) {
		progress.setProgressOf(f, i);
		if (i >= 990) {
			// El hilo ha terminado y la conversion no es concurrente
			parent.wakeUp();
		}
	}

	/**
	 * Recibe informacion del motor de conversion y la muestra por pantalla
	 */
	public void sourceInfo(MultimediaInfo info) {
		// System.out.println("Source info >> " + info);
		YakiToOutput.message("Source >> " + info);
	}

}
