package net.yakiboo.yakito;

import it.sauronsoftware.jave.AudioAttributes;
import it.sauronsoftware.jave.Encoder;
import it.sauronsoftware.jave.EncodingAttributes;
import it.sauronsoftware.jave.VideoAttributes;
import it.sauronsoftware.jave.VideoSize;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JProgressBar;

import net.yakiboo.yakito.config.YakiToConfig;
import net.yakiboo.yakito.io.YakiToOutput;
import net.yakiboo.yakito.utils.Stopwatch;
import net.yakiboo.yakito.view.TableUpdater;

/**
 * Clase que implementa la conversion de archivos multimedia
 * 
 * @author sgm
 * 
 */
public class YakiToConversion implements YakiToProgress {

	// Lista de archivos a convertir
	private LinkedList<File> files;
	// Lista de barras de progreso asociadas a los archivos
	private LinkedList<JProgressBar> bars;
	// Hilos de conversion
	private LinkedList<EncodeThread> threads;
	// Destino de los archivos convertidos
	private String destiny;
	// Tipo de conversion
	private boolean concurrent;
	// Contenedor multimedia
	private String format;

	// Atributos de audio
	AudioAttributes aatts = null;
	// Atributos de video
	VideoAttributes vatts = null;

	// Argumentos extra
	List<String> extraArgums = null;

	// Modos de redimensionado
	public enum Redim {
		NO, CUSTOM, OTHER
	};

	// Estado de ejecucion
	public enum State {
		RUNNING, PAUSED, STOPPED
	};

	private State status;

	// Redimensionado
	private Redim redimMode;
	// Otro redimensionado
	private String redim;

	// Modos de recodificacion
	public enum Mode {
		RECODE, COPY, DISABLED
	};

	// Modo de audio
	private Mode amode;
	// Modo de video
	private Mode vmode;

	// Codec de audio
	private String acodec;
	// Ratio de audio
	private int akbps;
	// Frecuencia de muestreo de audio
	private int afreq;
	// Canales de audio
	private int achannels;
	// Volumen de audio
	private int volume;

	// Codec de video
	private String vcodec;
	// Ratio de video
	private int vkbps;
	// Conservar calidad
	private boolean sameq;
	// Etiqueta de video
	private String vtag;
	// Frames por segundo
	private int fps;
	// Dimensiones de pantalla
	private int[] size = { 0, 0 };

	// Aditivos al nombre del fichero destino
	private String ads = "";

	// Refrescador de tabla
	private TableUpdater tableUpdater = null;

	// Cronometro
	private Stopwatch clock = new Stopwatch();
	// Tiempo de conversion acumulado
	private double time = 0;
	// Porcentaje completado
	private double percent = 0;

	/**
	 * Clase de calculo del tiempo restante de conversion
	 * 
	 * @author Sergio Garcia Mondaray
	 * 
	 */
	private class RemainingThread extends Thread {
		public boolean alive = false;

		/**
		 * Mata al hilo
		 */
		public void die() {
			// System.out.println("REMAININGTIME MUERE");
			alive = false;
		}

		/**
		 * Despierta al hilo
		 */
		public synchronized void wakeUp() {
			// System.out.println("REMAININGTIME SE DESPIERTA");
			if (!alive) {
				this.start();
			} else
				this.notify();
		}

		/**
		 * Duerme el hilo
		 */
		public synchronized void toSleep() {
			// System.out.println("REMAININGTIME SE DUERME");
			try {
				this.wait();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		@Override
		public void start() {
			alive = true;
			super.start();
		}

		@Override
		public void run() {
			while (alive) {
				clock.startClock();
				int remaining = (int) (time * (100 - percent) / percent);
				// System.out.println("Remaining: " + remaining);
				tableUpdater.setRemainingTime(remaining);
				try {
					this.sleep(1000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				clock.stopClock();
				time += clock.getResultInSeconds();
				// System.out.println("Han pasado " + time + " segundos");
			}
		}
	};

	private RemainingThread remainingTime;

	private YakiToConfig config;

	/**
	 * Constructor del objeto de conversion
	 * 
	 * @param tableUpdater
	 *            Modificador visual del progreso de los archivos que estan
	 *            siendo convertidos
	 * @param config
	 *            Configuracion de YakiTo
	 */
	public YakiToConversion(final TableUpdater tableUpdater, YakiToConfig config) {
		// Inicializamos la lista de archivos
		files = new LinkedList<File>();
		// Inicializamos la lista de barras de progreso
		bars = new LinkedList<JProgressBar>();
		// Inicializamos la lista de procesos de conversion
		threads = new LinkedList<EncodeThread>();
		// Inicializamos el objeto que refrescara la tabla
		this.tableUpdater = tableUpdater;
		// Inicializamos el hilo de tiempo restante
		this.remainingTime = new RemainingThread();
		// Inicializamos la configuracion
		this.config = config;
		// Inicializamos los atributos de audio y video
		vatts = new VideoAttributes();
		aatts = new AudioAttributes();
		// Argumentos extra
		this.extraArgums = new LinkedList<String>();
	}

	/**
	 * Obtiene la lista de archivos multimedia de la conversion
	 * 
	 * @return Lista de archivos a convertir
	 */
	public LinkedList<File> getFiles() {
		return files;
	}

	/**
	 * Inserta un fichero especificado en la lista de archivos a convertir
	 * 
	 * @param f
	 *            Archivo a convertir
	 * @return <code>true</code> si el archivo se inserta correctamente,
	 *         <code>false</code> si el archivo ya estaba en la lista.
	 */
	public boolean addFile(File f) {
		int pos = files.indexOf(f);
		boolean added = false;
		if (pos < 0) {
			added = true;
			files.add(f);
			JProgressBar bar = new JProgressBar();
			bar.setStringPainted(true);
			bars.add(bar);
			YakiToOutput.message("Añadido el archivo \"" + f.getAbsolutePath()
					+ "\" a la lista.");
		}
		return added;
	}

	/**
	 * Elimina un archivo de la lista
	 * 
	 * @param f
	 *            Archivo a eliminar
	 */
	public void removeFile(File f) {
		if (remove(f))
			YakiToOutput.message("Suprimido el archivo \""
					+ f.getAbsolutePath() + "\" de la lista.");
	}

	/**
	 * Elimina un determinado archivo de la lista de archivos a convertir
	 * 
	 * @param f
	 *            Archivo a eliminar de la lista
	 * @return Confirmacion de eliminacion correcta
	 */
	private boolean remove(File f) {
		int pos = files.indexOf(f);
		boolean removed = false;
		if (pos > -1) {
			files.remove(pos);
			bars.remove(pos);
			removed = true;
		}
		return removed;
	}

	/**
	 * Obtiene la barra de progreso asociada a un fichero especificado
	 * 
	 * @param f
	 *            archivo cuya barra de progreso queremos.
	 * @return Barra de progreso asociada al archivo.
	 */
	public JProgressBar getBarOfFile(File f) {
		int pos = files.indexOf(f);
		if (pos > -1)
			return bars.get(pos);
		else
			return null;
	}

	/**
	 * Da prioridad a un archivo sobre los demas. Lo que se traduce en
	 * adelantarlo en la lista una posición.
	 * 
	 * @param f
	 *            Archivo objetivo
	 */
	public void morePriority(File f) {
		int pos = files.indexOf(f);
		if (pos > 0) {
			files.remove(pos);
			JProgressBar b = bars.get(pos);
			bars.remove(pos);
			files.add(pos - 1, f);
			bars.add(pos - 1, b);
		}
	}

	/**
	 * Quita prioridad al archivo seleccionado. Lo que se traduce en desplazarlo
	 * hacia el final de la lista una posición.
	 * 
	 * @param f
	 *            Archivo objetivo
	 */
	public void lessPriority(File f) {
		int pos = files.indexOf(f);
		if (pos >= 0 && pos < files.size() - 1) {
			files.remove(pos);
			JProgressBar b = bars.get(pos);
			bars.remove(pos);
			files.add(pos + 1, f);
			bars.add(pos + 1, b);
		}
	}

	/**
	 * Obtiene el fichero objetivo en una cierta posicion
	 * 
	 * @param index
	 *            Posicion en la lista
	 * @return Archivo objetivo en dicha posicion
	 */
	public File getFileAt(int index) {
		return files.get(index);
	}

	/**
	 * Obtiene la posicion de un archivo en la lista de objetivos
	 * 
	 * @param f
	 *            Archivo objetivo
	 * @return Posicion en la lista
	 */
	public int getIndexOf(File f) {
		return files.indexOf(f);
	}

	/**
	 * Elimina todos los archivos objetivo.
	 */
	public void clearList() {
		files.clear();
		bars.clear();
		YakiToOutput.message("La lista de archivos a convertir se ha vaciado.");
	}

	/**
	 * Obtiene la ruta de directorios en la que se crean los archivos
	 * convertidos
	 * 
	 * @return Ruta de directorios
	 */
	public String getDestiny() {
		return destiny;
	}

	/**
	 * Establece la ruta de destino de los archivos creados
	 * 
	 * @param destiny
	 *            Ruta de directorios en la que crear los archivos multimedia
	 *            nuevos
	 */
	public void setDestiny(String destiny) {
		this.destiny = destiny;
	}

	/**
	 * Comprueba si el modo actual de conversion es la conversion paralela
	 * 
	 * @return Si la conversion es paralela o no
	 */
	public boolean isConcurrent() {
		return concurrent;
	}

	/**
	 * Establece el modo de conversion
	 * 
	 * @param concurrent
	 *            Modo paralelo
	 */
	public void setConcurrent(boolean concurrent) {
		this.concurrent = concurrent;
	}

	/**
	 * Obtiene el codec de video utilizado por la conversion
	 * 
	 * @return Codec de video
	 */
	public String getVCodec() {
		return vcodec;
	}

	/**
	 * Establece el codec de video a utilizar en la conversion
	 * 
	 * @param vcodec
	 *            Codec de video
	 */
	public void setVCodec(String vcodec) {
		vatts.setCodec(vcodec);
		this.vcodec = vcodec;
	}

	/**
	 * Obtiene el codec de audio utilizado por la conversion
	 * 
	 * @return Codec de audio utilizado
	 */
	public String getACodec() {
		return acodec;
	}

	/**
	 * Establece el codec d eaudio a utilizar en la conversion
	 * 
	 * @param acodec
	 *            Codec de audio
	 */
	public void setACodec(String acodec) {
		aatts.setCodec(acodec);
		this.acodec = acodec;
	}

	/**
	 * Obtiene el ratio de compresion del audio
	 * 
	 * @return Ratio de compresion de audio (en kb/s)
	 */
	public int getAkbps() {
		return akbps;
	}

	/**
	 * Establece el ratio de compresion del audio
	 * 
	 * @param akbps
	 *            Ratio de compresion (en kb/s)
	 */
	public void setAkbps(int akbps) {
		akbps = akbps * 1000;
		aatts.setBitRate(akbps);
		this.akbps = akbps;
	}

	/**
	 * Obtiene el ratio de compresion del stream de video
	 * 
	 * @return Ratio de compresion (en kb/s)
	 */
	public int getVkbps() {
		return vkbps;
	}

	/**
	 * Establece el ratio de compresion de video
	 * 
	 * @param vkbps
	 *            Ratio de compresion (en kb/s)
	 */
	public void setVkbps(int vkbps) {
		vatts.setBitRate(vkbps);
		this.vkbps = vkbps;
	}

	/**
	 * Obtiene la frecuencia de muestreo del flujo de audio
	 * 
	 * @return Frecuencia d emuestreo del audio
	 */
	public int getAfreq() {
		return afreq;
	}

	/**
	 * Establece la frecuencia de muestreo del flujo de audio
	 * 
	 * @param afreq
	 *            Frecuencia de muestreo del audio
	 */
	public void setAfreq(int afreq) {
		aatts.setSamplingRate(afreq);
		this.afreq = afreq;
	}

	/**
	 * Obtiene el numero de canales de audio del archivo resultante de la
	 * conversion
	 * 
	 * @return Numero de canales de audio
	 */
	public int getAchannels() {
		return achannels;
	}

	/**
	 * Establece el numero de canales de audio
	 * 
	 * @param achannels
	 *            Numero de canales de audio
	 */
	public void setAchannels(int achannels) {
		aatts.setChannels(achannels);
		this.achannels = achannels;
	}

	/**
	 * Obtiene el volumen de audio
	 * 
	 * @return Volumen de audio
	 */
	public int getVolume() {
		return volume;
	}

	/**
	 * Establece el volumen de audio
	 * 
	 * @param volume
	 *            Nuevo volumen (0 para dejar conservar el volumen original)
	 */
	public void setVolume(int volume) {
		aatts.setVolume(volume * 2 + 256);
		this.volume = volume;
	}

	/**
	 * Obtiene el modo de recodificacion de video
	 * 
	 * @return Modo de video
	 */
	public Mode getVmode() {
		return vmode;
	}

	/**
	 * Establece el modo de recodificacion de video
	 * 
	 * @param vmode
	 *            Modo de video
	 */
	public void setVmode(Mode vmode) {
		if (vmode == Mode.COPY)
			vatts.setCodec("copy");
		else if (vmode == Mode.DISABLED)
			vatts = null;
		this.vmode = vmode;
	}

	/**
	 * Limpia los atributos de conversion
	 */
	public void clean() {
		this.aatts = new AudioAttributes();
		this.vatts = new VideoAttributes();
		this.extraArgums = new LinkedList<String>();
	}

	/**
	 * Obtiene el modo de conversion de audio
	 * 
	 * @return Modo de audio
	 */
	public Mode getAmode() {
		return amode;
	}

	/**
	 * Establece el modo de conversion de audio
	 * 
	 * @param amode
	 *            Modo de audio
	 */
	public void setAmode(Mode amode) {
		if (amode == Mode.COPY) {
			// aatts = new AudioAttributes();
			aatts.setCodec("copy");
		} else if (amode == Mode.DISABLED) {
			aatts = null;
		}
		this.amode = amode;
	}

	/**
	 * Obtiene las dimensiones del video convertido
	 * 
	 * @return Dimension de pantalla
	 */
	public int[] getSize() {
		return size;
	}

	/**
	 * Establece el tamano de pantalla del video resultante
	 * 
	 * @param size
	 *            Dimensiones de la pantalla (en pixels)
	 */
	public void setSize(int[] size) {
		vatts.setSize(new VideoSize(size[0], size[1]));
		this.size = size;
	}

	/**
	 * Obtiene el numero de imagenes por segundo (frames per second)
	 * 
	 * @return fps del video convertido
	 */
	public int getFps() {
		return fps;
	}

	/**
	 * Establece el numero de fotogramas por segundo
	 * 
	 * @param fps
	 *            Imagenes por segundo
	 */
	public void setFps(int fps) {
		vatts.setFrameRate(fps);
		this.fps = fps;
	}

	/**
	 * Obtiene la etiqueta de video
	 * 
	 * @return Etiqueta de video
	 */
	public String getVtag() {
		return vtag;
	}

	/**
	 * Establece la etiqueta de video
	 * 
	 * @param vtag
	 *            etiqueta de video
	 */
	public void setVtag(String vtag) {
		vatts.setTag(vtag);
		this.vtag = vtag;
	}

	/**
	 * Comprueba si esta activada la opcion de conservar la calidad
	 * 
	 * @return Estado de activacion de la opcion de conservar calidad
	 */
	public boolean isSameq() {
		return sameq;
	}

	/**
	 * Establece si se activa la opcion de conservar calidad
	 * 
	 * @param sameq
	 *            Activado o desactivado
	 */
	public void setSameq(boolean sameq) {
		this.sameq = sameq;
	}

	/**
	 * Obtiene el modo de redimensionado del video
	 * 
	 * @return Modo de redimensionado
	 */
	public Redim getRedimMode() {
		return redimMode;
	}

	/**
	 * Establece el modo de redimensionado del video
	 * 
	 * @param redim
	 *            Modo de redimensionado
	 */
	public void setRedimMode(Redim redim) {
		this.redimMode = redim;
	}

	/**
	 * Obtiene la cadena de redimensionado
	 * 
	 * @return Cadena de redimensionado
	 */
	public String getRedim() {
		return redim;
	}

	/**
	 * Establece la cadena de redimensionado
	 * 
	 * @param redim
	 *            Cadena de redimensionado
	 */
	public void setRedim(String redim) {
		this.redim = redim;
	}

	@Override
	public String toString() {
		String s = "";
		s += "YakiToConversion:\n";
		s += "\tDESTINY: " + this.getDestiny() + "\n";
		s += "\tAMODE: " + this.amode.toString() + "\n";
		s += "\t\tACODEC: " + this.acodec + "\n";
		s += "\t\tAKBPS: " + this.akbps + "\n";
		s += "\t\tAFREQ: " + this.afreq + "\n";
		s += "\t\tACHANNELS: " + this.achannels + "\n";
		s += "\t\tAVOL: " + this.volume + "\n";
		s += "\tVMODE: " + this.vmode.toString() + "\n";
		s += "\t\tSAMEQ: " + this.sameq + "\n";
		s += "\t\tVCODEC: " + this.vcodec + "\n";
		s += "\t\tVKBPS: " + this.vkbps + "\n";
		s += "\t\tVTAG: " + this.vtag + "\n";
		s += "\t\tFPS: " + this.fps + "\n";
		s += "\t\tREDIM-MODE: " + this.redimMode + "\n";
		s += "\t\tREDIM: " + this.redim + "\n";
		s += "\t\tSIZE: " + this.size[0] + "x" + this.size[1] + "\n";
		return s;
	}

	/**
	 * Obtiene el fichero de destino correspondiente a un fichero multimedia
	 * origen
	 * 
	 * @param source
	 *            Archivo original
	 * @return Archivo de destino
	 */
	private File getTargetOf(File source) {
		File target = null;
		if (source != null) {
			String parent = this.getDestiny();
			String oldName = source.getName();
			int pos = oldName.lastIndexOf(".");
			String s = "";
			int cont = 0;
			do {
				if (pos > 0) {
					String pre = oldName.substring(0, pos);
					String suf = oldName.substring(pos);
					target = new File(parent
							+ System.getProperty("file.separator") + pre + ads
							+ s + "." + this.getFormat());
				}
				s = " (" + (++cont) + ")";
			} while (target.exists());
		}
		return target;
	}

	/**
	 * Obtiene los atributos de audio
	 * 
	 * @return Atributos de audio
	 */
	private AudioAttributes getAudioAttributes() {
		/*
		 * atts.setBitRate(this.akbps); atts.setChannels(this.achannels);
		 * atts.setCodec(this.acodec); atts.setSamplingRate(this.afreq);
		 * atts.setVolume(this.volume); return atts;
		 */
		return aatts;
	}

	/**
	 * Obtiene los atributos de video
	 * 
	 * @return Atributos de video
	 */
	private VideoAttributes getVideoAttributes() {
		/*
		 * atts.setBitRate(this.vkbps); atts.setCodec(this.vcodec);
		 * atts.setFrameRate(this.fps); atts.setTag(this.vtag); if
		 * (this.redimMode == redimMode.CUSTOM) { VideoSize vs = new
		 * VideoSize(size[0], size[1]); atts.setSize(vs); } return atts;
		 */
		return vatts;
	}

	private EncodingAttributes getEncodingAttributes() {
		EncodingAttributes atts = new EncodingAttributes();
		atts.setAudioAttributes(getAudioAttributes());
		atts.setVideoAttributes(getVideoAttributes());
		atts.setFormat(format);
		return atts;
	}

	/**
	 * Anade un argumento a la conversion
	 * 
	 * @param argum
	 *            Argumento a anadir
	 */
	public void addExtraArgum(String argum) {
		this.extraArgums.add(argum);
	}

	/**
	 * Inicia la conversion
	 */
	public void startConversion() {
		// remainingTime.wakeUp();

		if (status == status.PAUSED) { // TODO Pausa en la conversion
			remainingTime.wakeUp();
			YakiToOutput.message("Conversión reanudada");
			if (concurrent) {
				for (int i = 0; i < files.size(); i++) {
					threads.get(i).cont();
				}
			} else {
				// TODO Conversion no concurrente
			}
		} else {
			remainingTime = new RemainingThread();
			remainingTime.start();
			YakiToOutput.message("Convirtiendo " + files.size()
					+ " archivos...");
			for (int i = 0; i < files.size(); i++) {
				this.setProgressOf(files.get(i), 0);
			}
			if (concurrent) {
				final YakiToConversion conversion = this;
				CustomThread conv = new CustomThread() {
					int restantes = files.size();

					@Override
					public void run() {
						for (int i = 0; i < files.size(); i++) {
							EncodeThread aux = new EncodeThread(
									getEncodingAttributes(), files.get(i),
									getTargetOf(files.get(i)), conversion,
									this, extraArgums, config);
							threads.add(aux);
							aux.start();
						}
						while (restantes > 0) {
							toSleep();
							restantes--;
						}
						conversion.finishConversion();
					}

				};
				conv.start();
			} else {
				final YakiToConversion conversion = this;

				CustomThread conv = new CustomThread() {
					int restantes = files.size();

					@Override
					public void run() {
						// Creamos los hilos de conversión
						for (int i = 0; i < files.size(); i++) {
							File target = getTargetOf(files.get(i));
							EncodeThread aux = new EncodeThread(
									getEncodingAttributes(), files.get(i),
									target, conversion, this, extraArgums,
									config);
							threads.add(aux);
						}
						// Ejecutamos los hilos de conversión
						for (int i = 0; i < files.size(); i++) {
							EncodeThread aux = threads.get(i);
							aux.start();
							toSleep();
						}
						conversion.finishConversion();
					}
				};

				conv.start();

			}
		}
		status = status.RUNNING;

	}

	/**
	 * Finaliza la conversion
	 */
	public void finishConversion() {
		remainingTime.die();
		status = status.STOPPED;
		percent = 0;
		tableUpdater.setStatusToIdle();
		// Eliminamos los hilos de conversion
		threads = new LinkedList<EncodeThread>();
	}

	/**
	 * Pausa la conversion
	 */
	public void pauseConversion() {
		remainingTime.toSleep();
		time += clock.getResultInSeconds();
		status = status.PAUSED;
		/*
		 * for (int i = 0; i < files.size(); i++) { threads.get(i).pause(); }
		 */
	}

	/**
	 * Detiene la conversion
	 */
	public void stopConversion() {
		// Detenemos los hilos
		for (EncodeThread e : threads) {
			e.kill();
		}
		for (File f : files) {
			this.setProgressOf(f, 0);
		}
		// Finalizamos
		finishConversion();
	}

	/**
	 * Obtiene el formato asociado al contenedor multimedia
	 * 
	 * @return Formato
	 */
	public String getFormat() {
		return format;
	}

	/**
	 * Establece el formato de conversion
	 * 
	 * @param format
	 */
	public void setFormat(String format) {
		this.format = format;
	}

	/**
	 * Obtiene los aditivos de renombrado de archivos
	 * 
	 * @return Aditivos
	 */
	public String getAds() {
		return ads;
	}

	/**
	 * Establece los aditivos de renombrado de archivos
	 * 
	 * @param ads
	 *            Aditivos
	 */
	public void setAds(String ads) {
		this.ads = ads;
	}

	@Override
	public void setProgressOf(File f, int value) {
		// TODO Auto-generated method stub
		double val = value / 10.0;
		bars.get(this.getIndexOf(f)).setValue((int) val);
		String text = "";
		if (val == 100) {
			text = config.getLanguage().getValueOf("progress_completed");
		} else {
			text = String.valueOf(val).substring(0,
					String.valueOf(val).indexOf(".") + 2)
					+ "%";
		}
		bars.get(this.getIndexOf(f)).setString(text);

		this.tableUpdater.updateProgressBars();
		// Actualizamos el progreso total
		int total = files.size();
		double completado = 0;
		for (int i = 0; i < files.size(); i++) {
			completado += bars.get(i).getPercentComplete() * 100;
		}
		tableUpdater.setTotalProgress((int) (completado / total));
		this.percent = completado / total;
	}

	/**
	 * Carga los ultimos parametros de conversion
	 * 
	 * @param cfg
	 *            Configuracion de YakiTo
	 */
	public void loadLastParameters(YakiToConfig cfg) {
		this.destiny = cfg.getLastDestiny().getAbsolutePath();
	}

	/**
	 * Comprueba si alguno de los archivos a convertir fue convertido
	 * previamente en la misma sesion
	 * 
	 * @return Valor de verdad
	 */
	public boolean someConvertedPreviously() {
		boolean some = false;
		for (JProgressBar bar : bars) {
			if (bar.getValue() == 100) {
				some = true;
				break;
			}
		}
		return some;
	}

	/**
	 * Comprueba si un archivo multimedia esta soportado por YakiTo
	 * 
	 * @param f
	 *            Archivo multimedia
	 * @return Valor de verdad
	 */
	public boolean isSupported(File f) {
		Encoder enc = new Encoder();
		return enc.isSupported(f);
	}

}

/**
 * Hilo padre de la conversion
 * 
 * @author Sergio Garcia Mondaray
 * 
 */
class CustomThread extends Thread {

	/**
	 * Despierta al hilo
	 */
	public synchronized void wakeUp() {
		this.notify();
	}

	/**
	 * Duerme al hilo
	 */
	public synchronized void toSleep() {
		try {
			this.wait();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

};
