/**
 * YakiToLanguage.java
 */

package net.yakiboo.yakito.language;

import java.io.File;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import net.yakiboo.yakito.config.YakiToConfig;
import net.yakiboo.yakito.io.YakiToFileSystem;
import net.yakiboo.yakito.io.YakiToXMLSyntaxException;
import net.yakiboo.yakito.io.YakiToOutput;
import net.yakiboo.yakito.io.YakiToXML;

/**
 * Clase que representa un idioma de la interfaz
 * 
 * @author Sergio Garcia Mondaray
 * 
 */
public class YakiToLanguage {

	/**
	 * Mapa de traducciones. Las claves son los identificadores de las cadenas
	 * de texto a ser traducidas. El valor de cada clave es el texto en el
	 * idioma correspondiente.
	 */
	private HashMap<String, String> strings;

	/**
	 * Nombre del idioma.
	 */
	private String name;

	/**
	 * Imagen identificativa del idioma.
	 */
	private ImageIcon icon;

	/**
	 * Archivo del sistema de ficheros que se corresponde con el idioma.
	 */
	private File path;

	/**
	 * Listado de idiomas disponibles
	 */
	private static List<YakiToLanguage> availableLanguages = null;

	/**
	 * Constructor del idioma
	 * 
	 * @param f
	 *            Archivo de idioma, por ejemplo, "Spanish.lang"
	 * @param name
	 *            Nombre del idioma
	 * @param icon
	 *            Imagen del idioma (bandera o similar)
	 */
	public YakiToLanguage(File f, String name, ImageIcon icon)
			throws YakiToXMLSyntaxException {
		// Inicializamos los atributos del idioma
		this.path = f;
		this.name = name;
		this.icon = icon;
		// Parseariamos el xml para obtener las cadenas
		if (f != null)
			strings = YakiToXML.parseSimpleXMLFile(f);
	}

	private YakiToLanguage(String name) {
		this.name = name;
	}

	/**
	 * Obtiene el valor de una clave del idioma. Estas claves representan las
	 * diferentes cadenas de texto del idioma, y se corresponden con los tags de
	 * los archivos XML de idioma.
	 * 
	 * @param key
	 *            Identificador de la cadena de texto del idioma
	 * @return Valor de dicha cadena para este idioma
	 */
	public String getValueOf(String key) {
		String val = strings.get(key);
		return val;
	}

	@Override
	public String toString() {
		String s = "YakiToLanguage \"" + this.name + "\"\n";
		if (strings == null) {
			s += "\tnull\n";
		} else {
			for (String key : strings.keySet()) {
				s += "\t" + key + "\t" + strings.get(key) + "\n";
			}
		}
		return s;
	}

	@Override
	public boolean equals(Object o) {
		boolean eq = o instanceof YakiToLanguage;
		if (eq)
			eq = ((YakiToLanguage) o).getName().equals(getName());
		return eq;
	}

	/**
	 * Obtiene el nombre del idioma
	 * 
	 * @return Nombre del idioma
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Devuelve la imagen del idioma, en formato idono
	 * 
	 * @return Imagen identificativa del idioma
	 */
	public Icon getIcon() {
		return this.icon;
	}

	/**
	 * Obtiene la ruta absoluta del archivo de idioma
	 * 
	 * @return Archivo de idioma del sistema de ficheros
	 */
	public File getPath() {
		return this.path;
	}

	/**
	 * Lista todos los idiomas disponibles para la interfaz grafica de usuario
	 * 
	 * @param config
	 *            Archivo de configuracion de YakiTo
	 * @return Lista de idiomas
	 */
	public static List<YakiToLanguage> availableLanguages()
			throws YakiToNoLanguagesException {
		if (availableLanguages != null) {
			return availableLanguages;
		} else {
			LinkedList<YakiToLanguage> languages = new LinkedList<YakiToLanguage>();
			File langLocation = YakiToFileSystem.getLangLocation();
			if (langLocation != null) {
				File[] filesInLangLocation = langLocation.listFiles();
				for (File f : filesInLangLocation) {
					if (f.getName().endsWith(".lang")) {
						String s = f.getAbsolutePath();
						s = s.substring(0, s.length() - 4);
						s += "png";
						File ic = new File(s);
						ImageIcon aux = null;
						if (ic.exists())
							aux = new ImageIcon(ic.getAbsolutePath());
						try {
							languages.add(new YakiToLanguage(f, f.getName()
									.replace(".lang", ""), aux));
						} catch (YakiToXMLSyntaxException ex) {
							// Se omite este idioma
						}
					}
				}

				if (languages.size() == 0) {
					throw new YakiToNoLanguagesException();
				} else {
					String s = "";
					for (YakiToLanguage lang : languages) {
						s += lang.getName() + ", ";
					}
					YakiToOutput.message("Encontrados " + languages.size()
							+ " idiomas: " + s);
				}
				availableLanguages = languages;
				return languages;
			} else {
				throw new YakiToNoLanguagesException();
			}
		}
	}

	/**
	 * Obtiene el objeto YakiToLanguage cuyo nombre es el especificado
	 * 
	 * @param langName
	 *            Nombre del idioma
	 * @return Idioma con el nombre especificado
	 */
	public static YakiToLanguage getLanguageByName(String langName)
			throws YakiToNoLanguagesException {
		return availableLanguages().get(
				availableLanguages.indexOf(new YakiToLanguage(langName)));
	}

}
